Package: stagePop
Type: Package
Title: Modelling the Population Dynamics of a Stage-Structured Species
        in Continuous Time
Version: 1.1-2
Date: 2022-05-05
Author: Helen Kettle
Maintainer: David Nutter <david.nutter@bioss.ac.uk>
Description: Provides facilities to implement and run population models of
    stage-structured species...
NeedsCompilation: no
URL: https://github.com/HelenKettle/StagePop
Suggests: R.rsp
VignetteBuilder: R.rsp
License: GPL-3 | file LICENSE
Imports: deSolve,PBSddesolve, grDevices, graphics
Collate: 'SolverOptions.R' 'addNamesToList.R' 'checkICs.R'
        'checkSolution.R' 'deathFunction.R' 'emigrationFunction.R'
        'developmentFunction.R' 'immigrationFunction.R'
        'durationFunction.R' 'reproductionFunction.R'
        'defaultRateFunctions.R' 'derivDede.R' 'derivPBS.R'
        'genericPlot.R' 'initConditions.R' 'namingVariables.R'
        'plotStrains.R' 'popModel.R' 'rateFuncCheck.R' 'rateFunction.R'
        'runStagePopExample.R' 'stagePop-package.R' 'sumStrains.R'
        'vecDivideFunc.R' 'vecToMatList.R'
Packaged: 2022-05-10 12:19:53 UTC; helen
Repository: CRAN
Date/Publication: 2022-05-10 13:50:02 UTC
RoxygenNote: 7.1.2
Built: R 4.6.0; ; 2026-04-25 22:34:44 UTC; unix
